@@ -1,21 +1,23 @@
-import functools
 import importlib
 import json
 import logging
+import warnings
 import os
 import time
 import uuid
+
 from dataclasses import dataclass
-from queue import Empty, Queue
+from queue import Queue
 from threading import Thread
-from typing import (TYPE_CHECKING, Any, Callable, Dict, List, Mapping,
-                    Optional, Sequence, Set, Sized, Tuple, Union)
+from typing import (Hashable, Iterator, TYPE_CHECKING, Any, Callable, Dict,
+                    List, Mapping, Optional, Sequence, Set,
+                    Sized, Tuple, Union, cast)
 
 import numpy
+import pandas as pd
 
 import qcodes
-from qcodes.dataset.descriptions.dependencies import (DependencyError,
-                                                      InterDependencies_)
+from qcodes.dataset.descriptions.dependencies import InterDependencies_
 from qcodes.dataset.descriptions.param_spec import ParamSpec, ParamSpecBase
 from qcodes.dataset.descriptions.rundescriber import RunDescriber
 from qcodes.dataset.descriptions.versioning.converters import (new_to_old,
@@ -45,13 +47,15 @@ from qcodes.dataset.sqlite.query_helpers import (VALUE, VALUES,
                                                  length, one,
                                                  select_one_where)
 from qcodes.instrument.parameter import _BaseParameter
-
+from qcodes.utils.deprecate import deprecate
+from qcodes.dataset.export_config import DataExportType, get_data_export_type, get_data_export_path, get_data_export_prefix, get_data_export_automatic
 from .data_set_cache import DataSetCache
 from .descriptions.versioning import serialization as serial
+from .subscriber import _Subscriber
 
 if TYPE_CHECKING:
     import pandas as pd
-
+    import xarray as xr
 
 
 log = logging.getLogger(__name__)
@@ -88,127 +92,15 @@ ParameterData = Dict[str, Dict[str, numpy.ndarray]]
 class CompletedError(RuntimeError):
     pass
 
+
 class DataLengthException(Exception):
     pass
 
+
 class DataPathException(Exception):
     pass
 
 
-class _Subscriber(Thread):
-    """
-    Class to add a subscriber to a :class:`.DataSet`. The subscriber gets called every
-    time an insert is made to the results_table.
-
-    The _Subscriber is not meant to be instantiated directly, but rather used
-    via the 'subscribe' method of the :class:`.DataSet`.
-
-    NOTE: A subscriber should be added *after* all parameters have been added.
-
-    NOTE: Special care shall be taken when using the *state* object: it is the
-    user's responsibility to operate with it in a thread-safe way.
-    """
-    def __init__(self,
-                 dataSet: 'DataSet',
-                 id_: str,
-                 callback: Callable[..., None],
-                 state: Optional[Any] = None,
-                 loop_sleep_time: int = 0,  # in milliseconds
-                 min_queue_length: int = 1,
-                 callback_kwargs: Optional[Mapping[str, Any]] = None
-                 ) -> None:
-        super().__init__()
-
-        self._id = id_
-
-        self.dataSet = dataSet
-        self.table_name = dataSet.table_name
-        self._data_set_len = len(dataSet)
-
-        self.state = state
-
-        self.data_queue: "Queue[Any]" = Queue()
-        self._queue_length: int = 0
-        self._stop_signal: bool = False
-        # convert milliseconds to seconds
-        self._loop_sleep_time = loop_sleep_time / 1000
-        self.min_queue_length = min_queue_length
-
-        if callback_kwargs is None or len(callback_kwargs) == 0:
-            self.callback = callback
-        else:
-            self.callback = functools.partial(callback, **callback_kwargs)
-
-        self.callback_id = f"callback{self._id}"
-        self.trigger_id = f"sub{self._id}"
-
-        conn = dataSet.conn
-
-        conn.create_function(self.callback_id, -1, self._cache_data_to_queue)
-
-        parameters = dataSet.get_parameters()
-        sql_param_list = ",".join([f"NEW.{p.name}" for p in parameters])
-        sql_create_trigger_for_callback = f"""
-        CREATE TRIGGER {self.trigger_id}
-            AFTER INSERT ON '{self.table_name}'
-        BEGIN
-            SELECT {self.callback_id}({sql_param_list});
-        END;"""
-        atomic_transaction(conn, sql_create_trigger_for_callback)
-
-        self.log = logging.getLogger(f"_Subscriber {self._id}")
-
-    def _cache_data_to_queue(self, *args: Any) -> None:
-        self.data_queue.put(args)
-        self._data_set_len += 1
-        self._queue_length += 1
-
-    def run(self) -> None:
-        self.log.debug("Starting subscriber")
-        self._loop()
-
-    @staticmethod
-    def _exhaust_queue(queue: "Queue[Any]") -> List[Any]:
-        result_list = []
-        while True:
-            try:
-                result_list.append(queue.get(block=False))
-            except Empty:
-                break
-        return result_list
-
-    def _call_callback_on_queue_data(self) -> None:
-        result_list = self._exhaust_queue(self.data_queue)
-        self.callback(result_list, self._data_set_len, self.state)
-
-    def _loop(self) -> None:
-        while True:
-            if self._stop_signal:
-                self._clean_up()
-                break
-
-            if self._queue_length >= self.min_queue_length:
-                self._call_callback_on_queue_data()
-                self._queue_length = 0
-
-            time.sleep(self._loop_sleep_time)
-
-            if self.dataSet.completed:
-                self._call_callback_on_queue_data()
-                break
-
-    def done_callback(self) -> None:
-        self._call_callback_on_queue_data()
-
-    def schedule_stop(self) -> None:
-        if not self._stop_signal:
-            self.log.debug("Scheduling stop")
-            self._stop_signal = True
-
-    def _clean_up(self) -> None:
-        self.log.debug("Stopped subscriber")
-
-
 class _BackgroundWriter(Thread):
     """
     Write the results from the DataSet's dataqueue in a new thread
@@ -233,7 +125,8 @@ class _BackgroundWriter(Thread):
             elif item['keys'] == 'finalize':
                 _WRITERS[self.path].active_datasets.remove(item['values'])
             else:
-                self.write_results(item['keys'], item['values'], item['table_name'])
+                self.write_results(
+                    item['keys'], item['values'], item['table_name'])
             self.queue.task_done()
 
     def write_results(self, keys: Sequence[str],
@@ -286,7 +179,8 @@ class DataSet(Sized):
                  specs: Optional[SpecsOrInterDeps] = None,
                  values: Optional[VALUES] = None,
                  metadata: Optional[Mapping[str, Any]] = None,
-                 shapes: Optional[Shapes] = None) -> None:
+                 shapes: Optional[Shapes] = None,
+                 in_memory_cache: bool = True) -> None:
         """
         Create a new :class:`.DataSet` object. The object can either hold a new run or
         an already existing run. If a ``run_id`` is provided, then an old run is
@@ -294,27 +188,31 @@ class DataSet(Sized):
 
         Args:
             path_to_db: path to the sqlite file on disk. If not provided, the
-              path will be read from the config.
+                path will be read from the config.
             run_id: provide this when loading an existing run, leave it
-              as None when creating a new run
+                as None when creating a new run
             conn: connection to the DB; if provided and ``path_to_db`` is
-              provided as well, then a ``ValueError`` is raised (this is to
-              prevent the possibility of providing a connection to a DB
-              file that is different from ``path_to_db``)
+                provided as well, then a ``ValueError`` is raised (this is to
+                prevent the possibility of providing a connection to a DB
+                file that is different from ``path_to_db``)
             exp_id: the id of the experiment in which to create a new run.
-              Ignored if ``run_id`` is provided.
+                Ignored if ``run_id`` is provided.
             name: the name of the dataset. Ignored if ``run_id`` is provided.
             specs: paramspecs belonging to the dataset or an ``InterDependencies_``
-              object that describes the dataset. Ignored if ``run_id`` is provided.
+                object that describes the dataset. Ignored if ``run_id``
+                is provided.
             values: values to insert into the dataset. Ignored if ``run_id`` is
-              provided.
+                provided.
             metadata: metadata to insert into the dataset. Ignored if ``run_id``
-              is provided.
+                is provided.
             shapes:
                 An optional dict from names of dependent parameters to the shape
                 of the data captured as a list of integers. The list is in the
                 same order as the interdependencies or paramspecs provided.
                 Ignored if ``run_id`` is provided.
+            in_memory_cache: Should measured data be keep in memory
+                and available as part of the `dataset.cache` object.
+
         """
         self.conn = conn_from_dbpath_or_conn(conn, path_to_db)
 
@@ -324,6 +222,8 @@ class DataSet(Sized):
         #: In memory representation of the data in the dataset.
         self.cache: DataSetCache = DataSetCache(self)
         self._results: List[Dict[str, VALUE]] = []
+        self._in_memory_cache = in_memory_cache
+        self._export_path: Optional[str] = None
 
         if run_id is not None:
             if not run_exists(self.conn, run_id):
@@ -621,14 +521,6 @@ class DataSet(Sized):
         self.conn.close()
         self.conn = connect(self.path_to_db, self._debug)
 
-    def add_parameter(self, spec: ParamSpec) -> None:
-        """
-        Old method; don't use it.
-        """
-        raise NotImplementedError('This method has been removed. '
-                                  'Please use DataSet.set_interdependencies '
-                                  'instead.')
-
     def set_interdependencies(self,
                               interdeps: InterDependencies_,
                               shapes: Shapes = None) -> None:
@@ -763,7 +655,8 @@ class DataSet(Sized):
         if start_bg_writer:
             writer_status.write_in_background = True
             if writer_status.bg_writer is None:
-                writer_status.bg_writer = _BackgroundWriter(writer_status.data_write_queue, self.conn)
+                writer_status.bg_writer = _BackgroundWriter(
+                    writer_status.data_write_queue, self.conn)
             if not writer_status.bg_writer.is_alive():
                 writer_status.bg_writer.start()
         else:
@@ -836,7 +729,8 @@ class DataSet(Sized):
         writer_status = self._writer_status
 
         if writer_status.write_in_background:
-            writer_status.data_write_queue.put({'keys': 'finalize', 'values': self.run_id})
+            writer_status.data_write_queue.put(
+                {'keys': 'finalize', 'values': self.run_id})
             while self.run_id in writer_status.active_datasets:
                 time.sleep(self.background_sleep_time)
         else:
@@ -929,6 +823,88 @@ class DataSet(Sized):
         return get_parameter_data(self.conn, self.table_name,
                                   valid_param_names, start, end)
 
+    @staticmethod
+    def _parameter_data_identical(param_dict_a: Dict[str, numpy.ndarray],
+                                  param_dict_b: Dict[str, numpy.ndarray]) -> bool:
+
+        try:
+            numpy.testing.assert_equal(param_dict_a, param_dict_b)
+        except AssertionError:
+            return False
+
+        return True
+
+    def _same_setpoints(self, datadict: ParameterData) -> bool:
+
+        def _get_setpoints(dd: ParameterData) -> Iterator[Dict[str, numpy.ndarray]]:
+
+            for dep_name, param_dict in dd.items():
+                out = {
+                    name: vals for name, vals in param_dict.items() if name != dep_name
+                }
+                yield out
+
+        sp_iterator = _get_setpoints(datadict)
+
+        try:
+            first = next(sp_iterator)
+        except StopIteration:
+            return True
+
+        return all(self._parameter_data_identical(first, rest) for rest in sp_iterator)
+
+    def to_pandas_dataframe_dict(self,
+                                 *params: Union[str,
+                                                ParamSpec,
+                                                _BaseParameter],
+                                 start: Optional[int] = None,
+                                 end: Optional[int] = None) ->\
+            Dict[str, "pd.DataFrame"]:
+        """
+        Returns the values stored in the :class:`.DataSet` for the specified parameters
+        and their dependencies as a dict of :py:class:`pandas.DataFrame` s
+        Each element in the dict is indexed by the names of the requested
+        parameters.
+
+        Each DataFrame contains a column for the data and is indexed by a
+        :py:class:`pandas.MultiIndex` formed from all the setpoints
+        of the parameter.
+
+        If no parameters are supplied data will be be
+        returned for all parameters in the :class:`.DataSet` that are not them self
+        dependencies of other parameters.
+
+        If provided, the start and end arguments select a range of results
+        by result count (index). If the range is empty - that is, if the end is
+        less than or equal to the start, or if start is after the current end
+        of the :class:`.DataSet` – then a dict of empty :py:class:`pandas.DataFrame` s is
+        returned.
+
+        Args:
+            *params: string parameter names, QCoDeS Parameter objects, and
+                ParamSpec objects. If no parameters are supplied data for
+                all parameters that are not a dependency of another
+                parameter will be returned.
+            start: start value of selection range (by result count); ignored
+                if None
+            end: end value of selection range (by results count); ignored if
+                None
+
+        Returns:
+            Dictionary from requested parameter names to
+            :py:class:`pandas.DataFrame` s with the requested parameter as
+            a column and a indexed by a :py:class:`pandas.MultiIndex` formed
+            by the dependencies.
+        """
+        datadict = self.get_parameter_data(*params,
+                                           start=start,
+                                           end=end)
+        dfs_dict = self._load_to_dataframe_dict(datadict)
+        return dfs_dict
+
+    @deprecate(reason='This method will be removed due to inconcise naming, please '
+               'use the renamed method to_pandas_dataframe_dict',
+               alternative='to_pandas_dataframe_dict')
     def get_data_as_pandas_dataframe(self,
                                      *params: Union[str,
                                                     ParamSpec,
@@ -972,11 +948,231 @@ class DataSet(Sized):
             a column and a indexed by a :py:class:`pandas.MultiIndex` formed
             by the dependencies.
         """
+        return self.to_pandas_dataframe_dict(*params, start=start, end=end)
+
+    def to_pandas_dataframe(self,
+                            *params: Union[str,
+                                           ParamSpec,
+                                           _BaseParameter],
+                            start: Optional[int] = None,
+                            end: Optional[int] = None) -> "pd.DataFrame":
+        """
+        Returns the values stored in the :class:`.DataSet` for the specified parameters
+        and their dependencies as a concatenated :py:class:`pandas.DataFrame` s
+
+        The DataFrame contains a column for the data and is indexed by a
+        :py:class:`pandas.MultiIndex` formed from all the setpoints
+        of the parameter.
+
+        If no parameters are supplied data will be be
+        returned for all parameters in the :class:`.DataSet` that are not them self
+        dependencies of other parameters.
+
+        If provided, the start and end arguments select a range of results
+        by result count (index). If the range is empty - that is, if the end is
+        less than or equal to the start, or if start is after the current end
+        of the :class:`.DataSet` – then a dict of empty :py:class:`pandas.DataFrame` s is
+        returned.
+
+        Args:
+            *params: string parameter names, QCoDeS Parameter objects, and
+                ParamSpec objects. If no parameters are supplied data for
+                all parameters that are not a dependency of another
+                parameter will be returned.
+            start: start value of selection range (by result count); ignored
+                if None
+            end: end value of selection range (by results count); ignored if
+                None
+
+        Returns:
+            :py:class:`pandas.DataFrame` s with the requested parameter as
+            a column and a indexed by a :py:class:`pandas.MultiIndex` formed
+            by the dependencies.
+
+        Example:
+            Return a pandas DataFrame with
+                df = ds.to_pandas_dataframe()
+        """
         datadict = self.get_parameter_data(*params,
                                            start=start,
                                            end=end)
-        dfs = self._load_to_dataframes(datadict)
-        return dfs
+        return self._load_to_concatenated_dataframe(datadict)
+
+    def _load_to_concatenated_dataframe(self, datadict: ParameterData
+                                        ) -> "pd.DataFrame":
+        import pandas as pd
+
+        if not self._same_setpoints(datadict):
+            warnings.warn(
+                "Independent parameter setpoints are not equal. "
+                "Check concatenated output carefully. Please "
+                "consider using `to_pandas_dataframe_dict` to export each "
+                "independent parameter to its own dataframe."
+            )
+
+        dfs_dict = self._load_to_dataframe_dict(datadict)
+        df = pd.concat(list(dfs_dict.values()), axis=1)
+
+        return df
+
+    def to_xarray_dataarray_dict(self,
+                                 *params: Union[str,
+                                                ParamSpec,
+                                                _BaseParameter],
+                                 start: Optional[int] = None,
+                                 end: Optional[int] = None) -> \
+            Dict[str, "xr.DataArray"]:
+        """
+        Returns the values stored in the :class:`.DataSet` for the specified parameters
+        and their dependencies as a dict of :py:class:`xr.DataArray` s
+        Each element in the dict is indexed by the names of the requested
+        parameters.
+
+        If no parameters are supplied data will be be
+        returned for all parameters in the :class:`.DataSet` that are not them self
+        dependencies of other parameters.
+
+        If provided, the start and end arguments select a range of results
+        by result count (index). If the range is empty - that is, if the end is
+        less than or equal to the start, or if start is after the current end
+        of the :class:`.DataSet` – then a dict of empty :py:class:`xr.DataArray` s is
+        returned.
+
+        The dependent parameters of the Dataset are normally used as coordinates of the
+        XArray dataframe. However if non unique values are found for the dependent parameter
+        values we will fall back to using an index as coordinates.
+
+        Args:
+            *params: string parameter names, QCoDeS Parameter objects, and
+                ParamSpec objects. If no parameters are supplied data for
+                all parameters that are not a dependency of another
+                parameter will be returned.
+            start: start value of selection range (by result count); ignored
+                if None
+            end: end value of selection range (by results count); ignored if
+                None
+
+        Returns:
+            Dictionary from requested parameter names to :py:class:`xr.DataArray` s
+            with the requested parameter(s) as a column(s) and coordinates
+            formed by the dependencies.
+
+        Example:
+            Return a dict of xr.DataArray with
+
+                dataarray_dict = ds.to_xarray_dataarray_dict()
+        """
+        data = self.get_parameter_data(*params,
+                                       start=start,
+                                       end=end)
+        datadict = self._load_to_xarray_dataarray_dict(data)
+
+        for dataarray in datadict.values():
+            self._add_metadata_to_xarray(dataarray)
+        return datadict
+
+    def to_xarray_dataset(self, *params: Union[str,
+                                               ParamSpec,
+                                               _BaseParameter],
+                          start: Optional[int] = None,
+                          end: Optional[int] = None) -> "xr.Dataset":
+        """
+        Returns the values stored in the :class:`.DataSet` for the specified parameters
+        and their dependencies as a :py:class:`xr.Dataset` object.
+
+        If no parameters are supplied data will be be
+        returned for all parameters in the :class:`.DataSet` that are not then self
+        dependencies of other parameters.
+
+        If provided, the start and end arguments select a range of results
+        by result count (index). If the range is empty - that is, if the end is
+        less than or equal to the start, or if start is after the current end
+        of the :class:`.DataSet` – then a empty :py:class:`xr.Dataset` s is
+        returned.
+
+        The dependent parameters of the Dataset are normally used as coordinates of the
+        XArray dataframe. However if non unique values are found for the dependent parameter
+        values we will fall back to using an index as coordinates.
+
+        Args:
+            *params: string parameter names, QCoDeS Parameter objects, and
+                ParamSpec objects. If no parameters are supplied data for
+                all parameters that are not a dependency of another
+                parameter will be returned.
+            start: start value of selection range (by result count); ignored
+                if None
+            end: end value of selection range (by results count); ignored if
+                None
+
+        Returns:
+            :py:class:`xr.Dataset` with the requested parameter(s) data as
+            :py:class:`xr.DataArray` s and coordinates formed by the dependencies.
+
+        Example:
+            Return a concatenated xr.Dataset with
+
+                xds = ds.to_xarray_dataset()
+        """
+        data = self.get_parameter_data(*params,
+                                       start=start,
+                                       end=end)
+
+        return self._load_to_xarray_dataset(data)
+
+    def _load_to_xarray_dataset(self, data: ParameterData) -> "xr.Dataset":
+        import xarray as xr
+
+        if not self._same_setpoints(data):
+            warnings.warn(
+                "Independent parameter setpoints are not equal. "
+                "Check concatenated output carefully. Please "
+                "consider using `to_xarray_dataarray_dict` to export each "
+                "independent parameter to its own datarray."
+            )
+
+        data_xrdarray_dict = self._load_to_xarray_dataarray_dict(data)
+
+        # Casting Hashable for the key type until python/mypy#1114
+        # and python/typing#445 are resolved.
+        xrdataset = xr.Dataset(
+            cast(Dict[Hashable, xr.DataArray], data_xrdarray_dict))
+
+        for dim in xrdataset.dims:
+            if "index" != dim:
+                paramspec_dict = self.paramspecs[str(dim)]._to_dict()
+                xrdataset.coords[str(dim)].attrs.update(paramspec_dict.items())
+
+        self._add_metadata_to_xarray(xrdataset)
+
+        return xrdataset
+
+    def _add_metadata_to_xarray(
+            self,
+            xrdataset: Union["xr.Dataset", "xr.DataArray"]
+    ) -> None:
+        xrdataset.attrs.update({
+            "ds_name": self.name,
+            "sample_name": self.sample_name,
+            "exp_name": self.exp_name,
+            "snapshot": self.snapshot_raw or "null",
+            "guid": self.guid,
+            "run_timestamp": self.run_timestamp() or "",
+            "completed_timestamp": self.completed_timestamp() or "",
+            "captured_run_id": self.captured_run_id,
+            "captured_counter": self.captured_counter,
+            "run_id": self.run_id,
+            "run_description": serial.to_json_for_storage(self.description)
+        })
+        if self.run_timestamp_raw is not None:
+            xrdataset.attrs["run_timestamp_raw"] = self.run_timestamp_raw
+        if self.completed_timestamp_raw is not None:
+            xrdataset.attrs[
+                "completed_timestamp_raw"] = self.completed_timestamp_raw
+        if len(self._metadata) > 0:
+            xrdataset.attrs['extra_metadata'] = {}
+
+            for metadata_tag, metadata in self._metadata.items():
+                xrdataset.attrs['extra_metadata'][metadata_tag] = metadata
 
     @staticmethod
     def _data_to_dataframe(data: Dict[str, numpy.ndarray], index: Union["pd.Index", "pd.MultiIndex"]) -> "pd.DataFrame":
@@ -1017,13 +1213,37 @@ class DataSet(Sized):
                 names=keys[1:])
         return index
 
-    def _load_to_dataframes(self, datadict: ParameterData) -> Dict[str, "pd.DataFrame"]:
+    def _load_to_dataframe_dict(self, datadict: ParameterData) -> Dict[str, "pd.DataFrame"]:
         dfs = {}
         for name, subdict in datadict.items():
             index = self._generate_pandas_index(subdict)
             dfs[name] = self._data_to_dataframe(subdict, index)
         return dfs
 
+    def _load_to_xarray_dataarray_dict(self,
+                                       datadict: Dict[str, Dict[str, numpy.ndarray]]) -> \
+            Dict[str, "xr.DataArray"]:
+        import xarray as xr
+
+        data_xrdarray_dict: Dict[str, xr.DataArray] = {}
+
+        for name, subdict in datadict.items():
+            index = self._generate_pandas_index(subdict)
+            if index is not None and len(index.unique()) != len(index):
+                for _name in subdict:
+                    data_xrdarray_dict[_name] = self._data_to_dataframe(
+                        subdict, index).reset_index().to_xarray()[_name]
+                    paramspec_dict = self.paramspecs[_name]._to_dict()
+                    data_xrdarray_dict[_name].attrs.update(paramspec_dict.items())
+            else:
+                xrdarray: xr.DataArray = self._data_to_dataframe(
+                    subdict, index).to_xarray()[name]
+                data_xrdarray_dict[name] = xrdarray
+                paramspec_dict = self.paramspecs[name]._to_dict()
+                xrdarray.attrs.update(paramspec_dict.items())
+
+        return data_xrdarray_dict
+
     def write_data_to_text_file(self, path: str,
                                 single_file: bool = False,
                                 single_file_name: Optional[str] = None) -> None:
@@ -1053,6 +1273,8 @@ class DataSet(Sized):
             single_file: If true, merges the data of same length of multiple
                          dependent parameters to a single file.
             single_file_name: User defined name for the data to be concatenated.
+                              If no extension is passed (.dat, .csv or .txt),
+                              .dat is automatically appended.
 
         Raises:
             DataLengthException: If the data of multiple parameters have not same
@@ -1061,7 +1283,7 @@ class DataSet(Sized):
                                in a single file but no filename provided.
         """
         import pandas as pd
-        dfdict = self.get_data_as_pandas_dataframe()
+        dfdict = self.to_pandas_dataframe_dict()
         dfs_to_save = list()
         for parametername, df in dfdict.items():
             if not single_file:
@@ -1075,11 +1297,13 @@ class DataSet(Sized):
                 raise DataLengthException("You cannot concatenate data " +
                                           "with different length to a " +
                                           "single file.")
-            if single_file_name == None:
+            if single_file_name is None:
                 raise DataPathException("Please provide the desired file name " +
                                         "for the concatenated data.")
             else:
-                dst = os.path.join(path, f'{single_file_name}.dat')
+                if not single_file_name.lower().endswith(('.dat', '.csv', '.txt')):
+                    single_file_name = f'{single_file_name}.dat'
+                dst = os.path.join(path, single_file_name)
                 df_to_save = pd.concat(dfs_to_save, axis=1)
                 df_to_save.to_csv(path_or_buf=dst, header=False, sep='\t')
 
@@ -1178,7 +1402,7 @@ class DataSet(Sized):
         Enqueue the results into self._results
 
         Before we can enqueue the results, all values of the results dict
-        must have the same length. We enqueue each parameter tree seperately,
+        must have the same length. We enqueue each parameter tree separately,
         effectively mimicking making one call to add_results per parameter
         tree.
 
@@ -1191,22 +1415,37 @@ class DataSet(Sized):
 
         toplevel_params = (set(interdeps.dependencies)
                            .intersection(set(result_dict)))
+        if self._in_memory_cache:
+            new_results: Dict[str, Dict[str, numpy.ndarray]] = {}
         for toplevel_param in toplevel_params:
             inff_params = set(interdeps.inferences.get(toplevel_param, ()))
             deps_params = set(interdeps.dependencies.get(toplevel_param, ()))
             all_params = (inff_params
                           .union(deps_params)
                           .union({toplevel_param}))
-            res_dict: Dict[str, VALUE] = {}  # the dict to append to _results
+
+            if self._in_memory_cache:
+                new_results[toplevel_param.name] = {}
+                new_results[toplevel_param.name][toplevel_param.name] = self._reshape_array_for_cache(
+                    toplevel_param,
+                    result_dict[toplevel_param]
+                )
+                for param in all_params:
+                    if param is not toplevel_param:
+                        new_results[toplevel_param.name][param.name] = self._reshape_array_for_cache(
+                            param,
+                            result_dict[param]
+                        )
+
             if toplevel_param.type == 'array':
                 res_list = self._finalize_res_dict_array(
                     result_dict, all_params)
             elif toplevel_param.type in ('numeric', 'text', 'complex'):
                 res_list = self._finalize_res_dict_numeric_text_or_complex(
-                               result_dict, toplevel_param,
-                               inff_params, deps_params)
+                    result_dict, toplevel_param,
+                    inff_params, deps_params)
             else:
-                res_dict = {ps.name: result_dict[ps] for ps in all_params}
+                res_dict: Dict[str, VALUE] = {ps.name: result_dict[ps] for ps in all_params}
                 res_list = [res_dict]
             self._results += res_list
 
@@ -1218,6 +1457,36 @@ class DataSet(Sized):
         if standalones:
             stdln_dict = {st: result_dict[st] for st in standalones}
             self._results += self._finalize_res_dict_standalones(stdln_dict)
+            if self._in_memory_cache:
+                for st in standalones:
+                    new_results[st.name] = {
+                        st.name: self._reshape_array_for_cache(st, result_dict[st])
+                    }
+
+        if self._in_memory_cache:
+            self.cache.add_data(new_results)
+
+    @staticmethod
+    def _reshape_array_for_cache(
+            param: ParamSpecBase,
+            param_data: numpy.ndarray
+    ) -> numpy.ndarray:
+        """
+        Shape cache data so it matches data read from database.
+        This means:
+
+        - Add an extra singleton dim to array data
+        - flatten non array data into a linear array.
+        """
+        param_data = numpy.atleast_1d(param_data)
+        if param.type == "array":
+            new_data = numpy.reshape(
+                param_data,
+                (1,) + param_data.shape
+            )
+        else:
+            new_data = param_data.ravel()
+        return new_data
 
     @staticmethod
     def _finalize_res_dict_array(
@@ -1363,6 +1632,94 @@ class DataSet(Sized):
             log.debug(f"Waiting for write queue to empty.")
             writer_status.data_write_queue.join()
 
+    def _export_file_name(self, prefix: str, export_type: DataExportType) -> str:
+        """Get export file name"""
+        extension = export_type.value
+        return f"{prefix}{self.run_id}.{extension}"
+
+    def _export_as_netcdf(self, path: str, file_name: str) -> str:
+        """Export data as netcdf to a given path with file prefix"""
+        file_path = os.path.join(path, file_name)
+        xarr_dataset = self.to_xarray_dataset()
+        xarr_dataset.to_netcdf(path=file_path)
+        return path
+
+    def _export_as_csv(self, path: str, file_name: str) -> str:
+        """Export data as csv to a given path with file prefix"""
+        self.write_data_to_text_file(path=path, single_file=True, single_file_name=file_name)
+        return os.path.join(path, file_name)
+
+    def _export_data(self,
+                     export_type: DataExportType,
+                     path: Optional[str] = None,
+                     prefix: Optional[str] = None
+                     ) -> Optional[str]:
+        """Export data to disk with file name {prefix}{run_id}.{ext}.
+        Values for the export type, path and prefix can also be set in the qcodes
+        "dataset" config.
+
+        Args:
+            export_type: Data export type, e.g. DataExportType.NETCDF
+            path: Export path, defaults to value set in config
+            prefix: File prefix, e.g. "qcodes_", defaults to value set in config.
+
+        Returns:
+            str: Path file was saved to, returns None if no file was saved.
+        """
+        # Set defaults to values in config if the value was not set
+        # (defaults to None)
+        path = path if path is not None else get_data_export_path()
+        prefix = prefix if prefix is not None else get_data_export_prefix()
+
+        if DataExportType.NETCDF == export_type:
+            file_name = self._export_file_name(
+                prefix=prefix, export_type=DataExportType.NETCDF)
+            return self._export_as_netcdf(path=path, file_name=file_name)
+
+        elif DataExportType.CSV == export_type:
+            file_name = self._export_file_name(
+                prefix=prefix, export_type=DataExportType.CSV)
+            return self._export_as_csv(path=path, file_name=file_name)
+
+        else:
+            return None
+
+    def export(self,
+               export_type: Optional[Union[DataExportType, str]] = None,
+               path: Optional[str] = None,
+               prefix: Optional[str] = None) -> None:
+        """Export data to disk with file name {prefix}{run_id}.{ext}.
+        Values for the export type, path and prefix can also be set in the "dataset"
+        section of qcodes config.
+
+        Args:
+            export_type: Data export type, e.g. "netcdf" or ``DataExportType.NETCDF``,
+                defaults to a value set in qcodes config
+            path: Export path, defaults to value set in config
+            prefix: File prefix, e.g. ``qcodes_``, defaults to value set in config.
+
+        Raises:
+            ValueError: If the export data type is not specified, raise an error
+        """
+        export_type = get_data_export_type(export_type)
+
+        if export_type is None:
+            raise ValueError(
+                "No data export type specified. Please set the export data type "
+                "by using ``qcodes.dataset.export_config.set_data_export_type`` or "
+                "give an explicit export_type when calling ``dataset.export`` manually."
+            )
+
+        self._export_path = self._export_data(
+            export_type=export_type,
+            path=path,
+            prefix=prefix
+        )
+
+    @property
+    def export_path(self) -> Optional[str]:
+        return self._export_path
+
 
 # public api
 def load_by_id(run_id: int, conn: Optional[ConnectionPlus] = None) -> DataSet:
@@ -1520,7 +1877,9 @@ def new_data_set(name: str,
                  specs: Optional[SPECS] = None,
                  values: Optional[VALUES] = None,
                  metadata: Optional[Any] = None,
-                 conn: Optional[ConnectionPlus] = None) -> DataSet:
+                 conn: Optional[ConnectionPlus] = None,
+                 in_memory_cache: bool = True,
+                 ) -> DataSet:
     """
     Create a new dataset in the currently active/selected database.
 
@@ -1533,6 +1892,8 @@ def new_data_set(name: str,
         specs: list of parameters to create this dataset with
         values: the values to associate with the parameters
         metadata: the metadata to associate with the dataset
+        in_memory_cache: Should measured data be keep in memory
+            and available as part of the `dataset.cache` object.
 
     Return:
         the newly created :class:`.DataSet`
@@ -1541,7 +1902,7 @@ def new_data_set(name: str,
     # in `Runner` to pass a connection from an existing `Experiment`.
     d = DataSet(path_to_db=None, run_id=None, conn=conn,
                 name=name, specs=specs, values=values,
-                metadata=metadata, exp_id=exp_id)
+                metadata=metadata, exp_id=exp_id, in_memory_cache=in_memory_cache)
 
     return d
 
