@@ -1,15 +1,17 @@
+from typing import Any
+
 from qcodes.instrument.visa import VisaInstrument
 from qcodes.utils import validators as vals
 import numpy as np
 
 
 class Weinschel_8320(VisaInstrument):
-    '''
+    """
     QCodes driver for the stepped attenuator
     Weinschel is formerly known as Aeroflex/Weinschel
-    '''
+    """
 
-    def __init__(self, name, address, **kwargs):
+    def __init__(self, name: str, address: str, **kwargs: Any):
         super().__init__(name, address, terminator='\r', **kwargs)
 
         self.add_parameter('attenuation', unit='dB',
